import geopandas as gpd
from shapely.geometry import Polygon, LineString
from shapely.ops import linemerge, unary_union, polygonize
import matplotlib.pyplot as plt
import random
import json
import numpy as np

polygon_min_x = None  # The left most point of the entire polygon


# Read a json file with relation data and send to response object
def get_relation(self, body_of_water: str):  # NB: implement body_of_water
    # Read relation from GeoJson file and extract all polygons
    geo_data = gpd.read_file("server/map/mjosa.geojson")
    polygon_data = geo_data[geo_data['geometry'].geom_type == 'Polygon']
    polygons = [Polygon(polygon.exterior) for polygon in polygon_data['geometry']]

    if len(polygons) <= 1:
        print("Failed to convert to polygons")
        return

    divided_map = []

    for polygon in polygons:
        cell_size = 0.04
        lines = create_grid(polygon, cell_size)
        lines.append(polygon.boundary)
        lines = unary_union(lines)
        lines = linemerge(lines)
        lines = list(polygonize(lines))

        divided_map.extend(combine_grid_with_poly(polygon, lines))

    tiles = [gpd.GeoDataFrame(geometry=[tile]) for tile in divided_map]

    sub_div_id = 0
    for tile in tiles:
        tile['sub_div_id'] = sub_div_id
        tile['sub_div_center'] = tile['geometry'].centroid.apply(lambda x: [x.x, x.y])
        sub_div_id += 1

    tiles_json = {'type': 'FeatureCollection', 'features': []}
    for tile in tiles:
        feature = {
            'type': 'Feature',
            'geometry': tile.geometry.__geo_interface__,
            'properties': {
                'sub_div_id': int(tile['sub_div_id'].iloc[0]),
                'sub_div_center': tile['sub_div_center'].tolist()
            }
        }
        tiles_json['features'].append(feature)

    self.send_response(200)
    self.send_header("Content-type", "application/json")
    self.end_headers()

    self.wfile.write(json.dumps(tiles_json).encode('utf-8'))



def create_grid(poly: Polygon, cell_size):
    # Retrieve bounds of the entire polygon
    bounds = poly.bounds

    min_x, min_y, max_x, max_y = bounds
    grid_lines = []

    # Horizontal lines
    y = min_y
    while y <= max_y:
        line = LineString([(min_x, y), (max_x, y)])
        grid_lines.append(line)
        y += cell_size

    # Vertical lines
    x = min_x
    while x <= max_x:
        line = LineString([(x, min_y), (x, max_y)])
        grid_lines.append(line)
        x += cell_size

    return grid_lines


def combine_grid_with_poly(polygon, grid):
    # Create an empty list to store tiles intersecting the polygon
    intersecting_tiles = []

    # Iterate through each grid line
    for line in grid:
        # Check if the line intersects with the polygon
        if line.intersects(polygon):
            # If the line intersects, find the intersection points
            intersection = line.intersection(polygon)
            # Add each line to the list
            intersecting_tiles.append(intersection)

    return intersecting_tiles
